program gsmGpsUdpDemo;

{$mode objfpc}{$H+}

uses
    {$IFDEF UNIX}{$IFDEF UseCThreads}
    cthreads,
    {$ENDIF}{$ENDIF}
    base64,
    blcksock,
    Classes,
    math,
    sysutils;

{$R *.res}

const
    htmlFname                   = 'gpspoints.html';

type
    udpRecT                     = record
        case byte of
            1:(
                latitude        : single;
                longitude       : single;
                speed           : smallint;
                course          : smallint;
                );
            2:(
                b               : array[1..12] of char;
                );
        end; // record

var

    buf                         : string;
    dist                        : double;
    htmlF                       : text;
    i                           : integer;
    oldLat                      : double    = 0.0;
    oldLon                      : double    = 0.0;
    s                           : string;
    udp                         : TUDPBlockSocket;
    udpRec                      : udpRecT;

  // ------------------------------
  // calculate distance between two long/lat points

  function calcDist(
        lat1                           : double;
        lon1                           : double;
        lat2                           : double;
        lon2                           : double
        )                              : double;

  begin

    calcDist := arccos(sin(degtorad(lat1))*sin(degtorad(lat2))+
                  cos(degtorad(lat1))*cos(degtorad(lat2))*
                  cos(degtorad(lon2)-degtorad(lon1)))*6371;
    CalcDist := CalcDist * 0.621371192; // convert from km to miles
    calcDist := calcDist * 5280         // for this program, I want feet returned

  end; // calcDist

begin

assign(htmlF, htmlFname);
if not fileExists(htmlFname) then begin
    rewrite(htmlF);
    close(htmlF);
    end;

udp := TUDPBlockSocket.create;

try
    udp.bind('0.0.0.0', '9999');                            // open udp port 9999
except
    writeln('bind failed');
    end;

while true do begin
    while (udp.WaitingData = 0) do begin
        sleep(500);
        end;

    s := udp.RecvPacket(1000);
    if udp.LastError = 0 then begin
        buf := decodeStringBase64(s);
        for i := 1 to length(buf) do
            udprec.b[i] := buf[i];
        dist := calcDist(oldLat, oldLon, udpRec.latitude, udpRec.longitude);
        writeln(format('%s: %10.6F %10.6F %3d MPH %3d Degrees dist: %10.4F',
            [formatdatetime('hh:nn:ss',now), udprec.latitude, udpRec.longitude,
            udpRec.speed, udpRec.course, dist]));
        oldLat := udpRec.latitude;
        oldLon := udpRec.longitude;
        if (dist > 20) then begin
            try
                append(htmlF);
                writeln(htmlF, format('new google.maps.LatLng(%10.6f,%10.6f),',
                [udprec.latitude, udprec.longitude]));
            finally
                close(htmlF);
                end; //try
            end;
        end;
    end; // while

end.

